"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.clear = exports.initialize = exports.CommandOptions = exports.defaultConfigFile = void 0;
var tslib_1 = require("tslib");
var commander_1 = tslib_1.__importDefault(require("commander"));
exports.defaultConfigFile = 'dtsgen.json';
var CommandOptions = (function () {
    function CommandOptions() {
        this.files = [];
        this.urls = [];
    }
    CommandOptions.prototype.isReadFromStdin = function () {
        return (this.stdin || (this.files.length === 0 && this.urls.length === 0));
    };
    return CommandOptions;
}());
exports.CommandOptions = CommandOptions;
var opts = new CommandOptions();
clear();
exports.default = opts;
function initialize(argv) {
    return parse(opts, argv);
}
exports.initialize = initialize;
function clear() {
    opts.configFile = undefined;
    opts.files = [];
    opts.urls = [];
    opts.stdin = undefined;
    opts.out = undefined;
    opts.target = undefined;
    opts.info = undefined;
    opts.outputAST = undefined;
}
exports.clear = clear;
function parse(o, argv) {
    var command = new commander_1.default.Command();
    function collectUrl(val, memo) {
        memo.push(val);
        return memo;
    }
    var pkg = require('../package.json');
    command
        .name(pkg.name)
        .version(pkg.version)
        .usage('[options] <file ... | file patterns using node-glob>')
        .option('-c, --config <file>', 'set configuration file path.')
        .option('--url <url>', 'input json schema from the url.', collectUrl, [])
        .option('--stdin', 'read stdin with other files or urls.')
        .option('-o, --out <file>', 'output filename.')
        .option('-t, --target <version>', "Specify ECMAScript target version: 'ES3', 'ES5', 'ES2015', 'ES2016', 'ES2017', 'ES2018', 'ES2019', 'ES2020', or 'ESNEXT' (default).")
        .option('--info', 'for developer mode. output loaded config and plugin details only.')
        .option('--output-ast', 'output TypeScript AST instead of d.ts file.')
        .on('--help', function () {
        console.log('');
        console.log('  Examples:');
        console.log('');
        console.log('    $ dtsgen --help');
        console.log('    $ dtsgen --out types.d.ts schema/**/*.schema.json');
        console.log('    $ cat schema1.json | dtsgen -c dtsgen.json');
        console.log('    $ dtsgen -o swaggerSchema.d.ts --url https://raw.githubusercontent.com/OAI/OpenAPI-Specification/master/schemas/v2.0/schema.json');
        console.log('    $ dtsgen -o petstore.d.ts --url https://raw.githubusercontent.com/OAI/OpenAPI-Specification/master/examples/v2.0/yaml/petstore.yaml');
        console.log('    $ dtsgen -c dtsgen-test.json --info');
    })
        .parse(argv);
    var res = command;
    o.configFile = res.config;
    o.files = command.args;
    o.urls = res.url;
    o.stdin = res.stdin;
    o.out = res.out;
    o.target = res.target;
    o.info = res.info;
    o.outputAST = res.outputAst;
    return command;
}
//# sourceMappingURL=commandOptions.js.map