"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.showConfig = exports.clearToDefault = exports.setConfig = void 0;
var tslib_1 = require("tslib");
var typescript_1 = tslib_1.__importDefault(require("typescript"));
var type_1 = require("./type");
var defaultConfig = {
    input: {
        files: [],
        urls: [],
        stdin: false,
    },
    outputAST: false,
    target: typescript_1.default.ScriptTarget.Latest,
    plugins: {},
};
var config = Object.assign({}, defaultConfig);
function setConfig(input) {
    config = Object.assign(config, input);
}
exports.setConfig = setConfig;
function clearToDefault() {
    setConfig(defaultConfig);
}
exports.clearToDefault = clearToDefault;
function showConfig(version, c) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, _b, _c, name_1, option, plugins, _d, _e, _f, name_2, option, p, e_1_1, plugins_1, plugins_1_1, p;
        var e_2, _g, e_1, _h, e_3, _j;
        return tslib_1.__generator(this, function (_k) {
            switch (_k.label) {
                case 0:
                    console.log('Version: ' + version);
                    console.log('ConfigFile: ' + c.configFile);
                    console.log();
                    console.log('Config:');
                    console.log('  input:');
                    if (c.input.files.length > 0) {
                        console.log('    files: ' + JSON.stringify(c.input.files));
                    }
                    if (c.input.urls.length > 0) {
                        console.log('    urls: ' + JSON.stringify(c.input.urls));
                    }
                    if (c.input.stdin) {
                        console.log('    stdin: true');
                    }
                    if (c.outputFile != null) {
                        console.log('  outputFile: ' + JSON.stringify(c.outputFile));
                    }
                    console.log('  target: ' + showScriptTarget(c.target));
                    if (c.outputAST) {
                        console.log('  outputAST: true');
                    }
                    console.log('  plugins:');
                    try {
                        for (_a = tslib_1.__values(Object.entries(c.plugins)), _b = _a.next(); !_b.done; _b = _a.next()) {
                            _c = tslib_1.__read(_b.value, 2), name_1 = _c[0], option = _c[1];
                            console.log("    " + name_1 + ": " + JSON.stringify(option));
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (_b && !_b.done && (_g = _a.return)) _g.call(_a);
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                    console.log();
                    plugins = [];
                    _k.label = 1;
                case 1:
                    _k.trys.push([1, 6, 7, 8]);
                    _d = tslib_1.__values(Object.entries(c.plugins)), _e = _d.next();
                    _k.label = 2;
                case 2:
                    if (!!_e.done) return [3, 5];
                    _f = tslib_1.__read(_e.value, 2), name_2 = _f[0], option = _f[1];
                    return [4, type_1.loadPlugin(name_2, option)];
                case 3:
                    p = _k.sent();
                    if (p != null) {
                        plugins.push(p);
                    }
                    _k.label = 4;
                case 4:
                    _e = _d.next();
                    return [3, 2];
                case 5: return [3, 8];
                case 6:
                    e_1_1 = _k.sent();
                    e_1 = { error: e_1_1 };
                    return [3, 8];
                case 7:
                    try {
                        if (_e && !_e.done && (_h = _d.return)) _h.call(_d);
                    }
                    finally { if (e_1) throw e_1.error; }
                    return [7];
                case 8:
                    console.log('Plugins: count=' + plugins.length);
                    try {
                        for (plugins_1 = tslib_1.__values(plugins), plugins_1_1 = plugins_1.next(); !plugins_1_1.done; plugins_1_1 = plugins_1.next()) {
                            p = plugins_1_1.value;
                            console.log("  " + p.meta.name + "@" + p.meta.version + ": " + p.meta.description);
                        }
                    }
                    catch (e_3_1) { e_3 = { error: e_3_1 }; }
                    finally {
                        try {
                            if (plugins_1_1 && !plugins_1_1.done && (_j = plugins_1.return)) _j.call(plugins_1);
                        }
                        finally { if (e_3) throw e_3.error; }
                    }
                    console.log();
                    return [2];
            }
        });
    });
}
exports.showConfig = showConfig;
function showScriptTarget(target) {
    switch (target) {
        case typescript_1.default.ScriptTarget.ES3:
            return 'ES3';
        case typescript_1.default.ScriptTarget.ES5:
            return 'ES5';
        case typescript_1.default.ScriptTarget.ES2015:
            return 'ES2015';
        case typescript_1.default.ScriptTarget.ES2016:
            return 'ES2016';
        case typescript_1.default.ScriptTarget.ES2017:
            return 'ES2017';
        case typescript_1.default.ScriptTarget.ES2018:
            return 'ES2018';
        case typescript_1.default.ScriptTarget.ES2019:
            return 'ES2019';
        case typescript_1.default.ScriptTarget.ES2020:
            return 'ES2020';
        case typescript_1.default.ScriptTarget.ESNext:
            return 'ESNext';
        default:
            return 'Latest';
    }
}
exports.default = config;
//# sourceMappingURL=config.js.map