"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var url = tslib_1.__importStar(require("url"));
var validateIdentifier_1 = require("./validateIdentifier");
var SchemaId = (function () {
    function SchemaId(inputId, parentIds) {
        this.inputId = inputId;
        var absoluteId = url.resolve('', inputId);
        if (parentIds) {
            parentIds.forEach(function (parent) {
                if (parent) {
                    absoluteId = url.resolve(parent, absoluteId);
                }
            });
        }
        if (absoluteId.indexOf('#') < 0) {
            absoluteId += '#';
        }
        if (absoluteId.indexOf('://') < 0 &&
            absoluteId[0] !== '/' &&
            absoluteId[0] !== '#') {
            absoluteId = '/' + absoluteId;
        }
        this.id = url.parse(absoluteId);
        this.absoluteId = this.id.href;
    }
    SchemaId.prototype.getAbsoluteId = function () {
        return this.absoluteId;
    };
    SchemaId.prototype.isEmpty = function () {
        return !!this.absoluteId;
    };
    SchemaId.prototype.isFetchable = function () {
        return /https?:\/\//.test(this.absoluteId);
    };
    SchemaId.prototype.getFileId = function () {
        return this.absoluteId.replace(/#.*$/, '#');
    };
    SchemaId.prototype.existsJsonPointerHash = function () {
        return this.absoluteId === '#' || /#\//.test(this.absoluteId);
    };
    SchemaId.prototype.getJsonPointerHash = function () {
        var m = /(#\/.*)$/.exec(this.absoluteId);
        if (m == null) {
            return '#';
        }
        return decodeURIComponent(m[1]);
    };
    SchemaId.prototype.toNames = function () {
        var uri = this.id;
        var ids = [];
        if (uri.host) {
            ids.push(decodeURIComponent(uri.host));
        }
        var addAllParts = function (path) {
            var paths = path.split('/');
            if (paths.length > 1 && paths[0] === '') {
                paths.shift();
            }
            paths.forEach(function (item) {
                ids.push(decodeURIComponent(item));
            });
        };
        if (uri.pathname) {
            addAllParts(uri.pathname);
        }
        if (uri.hash && uri.hash.length > 1) {
            addAllParts(uri.hash.substr(1));
        }
        return ids.map(validateIdentifier_1.toTypeName);
    };
    SchemaId.empty = new SchemaId('');
    return SchemaId;
}());
exports.default = SchemaId;
//# sourceMappingURL=schemaId.js.map