"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadPlugin = exports.parseFileContent = exports.readSchemaFromUrl = exports.readSchemasFromFile = exports.readSchemaFromStdin = exports.parseSchema = void 0;
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var js_yaml_1 = tslib_1.__importDefault(require("js-yaml"));
var path_1 = tslib_1.__importDefault(require("path"));
var utils_1 = require("../utils");
var schemaId_1 = tslib_1.__importDefault(require("./schemaId"));
var jsonSchema_1 = require("./jsonSchema");
function parseSchema(content, url) {
    var _a = jsonSchema_1.selectSchemaType(content), type = _a.type, openApiVersion = _a.openApiVersion;
    if (url != null) {
        jsonSchema_1.setId(type, content, url);
    }
    var id = jsonSchema_1.getId(type, content);
    return {
        type: type,
        openApiVersion: openApiVersion,
        id: id ? new schemaId_1.default(id) : schemaId_1.default.empty,
        content: content,
    };
}
exports.parseSchema = parseSchema;
function readSchemaFromStdin() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var data, content;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, utils_1.readStream(process.stdin)];
                case 1:
                    data = _a.sent();
                    content = parseFileContent(data);
                    return [2, parseSchema(content)];
            }
        });
    });
}
exports.readSchemaFromStdin = readSchemaFromStdin;
function readSchemasFromFile(pattern) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var files;
        var _this = this;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, utils_1.globFiles(pattern)];
                case 1:
                    files = _a.sent();
                    return [2, Promise.all(files.map(function (file) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                            var data, content;
                            return tslib_1.__generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4, fs_1.default.promises.readFile(file, {
                                            encoding: 'utf-8',
                                        })];
                                    case 1:
                                        data = _a.sent();
                                        content = parseFileContent(data);
                                        return [2, parseSchema(content)];
                                }
                            });
                        }); }))];
            }
        });
    });
}
exports.readSchemasFromFile = readSchemasFromFile;
function readSchemaFromUrl(url) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var data, content;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, utils_1.readUrl(url)];
                case 1:
                    data = _a.sent();
                    content = parseFileContent(data, url);
                    return [2, parseSchema(content, url)];
            }
        });
    });
}
exports.readSchemaFromUrl = readSchemaFromUrl;
function parseFileContent(content, filename) {
    var ext = filename ? path_1.default.extname(filename).toLowerCase() : '';
    var maybeYaml = ext === '.yaml' || ext === '.yml';
    try {
        if (maybeYaml) {
            return deepCopy(js_yaml_1.default.safeLoad(content));
        }
        else {
            return JSON.parse(content);
        }
    }
    catch (e) {
        if (maybeYaml) {
            return JSON.parse(content);
        }
        else {
            return deepCopy(js_yaml_1.default.safeLoad(content));
        }
    }
}
exports.parseFileContent = parseFileContent;
function deepCopy(obj) {
    return JSON.parse(JSON.stringify(obj));
}
function loadPlugin(name, option) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var mod, plugin;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!option) {
                        return [2, undefined];
                    }
                    return [4, Promise.resolve().then(function () { return tslib_1.__importStar(require(name)); })];
                case 1:
                    mod = _a.sent();
                    if (!('default' in mod)) {
                        console.warn("The plugin (" + name + ") is invalid module. That is not default export format.");
                        return [2, undefined];
                    }
                    plugin = mod.default;
                    if (plugin.preProcess != null && typeof plugin.preProcess !== 'function') {
                        console.warn("The plugin (" + name + ") is invalid module. The 'preProcess' is not a function.");
                        return [2, undefined];
                    }
                    if (plugin.postProcess != null &&
                        typeof plugin.postProcess !== 'function') {
                        console.warn("The plugin (" + name + ") is invalid module. The 'postProcess' is not a function.");
                        return [2, undefined];
                    }
                    return [2, plugin];
            }
        });
    });
}
exports.loadPlugin = loadPlugin;
//# sourceMappingURL=type.js.map