"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.globFiles = exports.readUrl = exports.readStream = void 0;
var tslib_1 = require("tslib");
var glob_1 = tslib_1.__importDefault(require("glob"));
var https_proxy_agent_1 = tslib_1.__importDefault(require("https-proxy-agent"));
var url_1 = require("url");
function readStream(stream, encoding) {
    if (encoding === void 0) { encoding = 'utf8'; }
    stream.setEncoding(encoding);
    return new Promise(function (resolve, reject) {
        var data = '';
        stream.on('data', function (chunk) { return (data += chunk); });
        stream.on('end', function () { return resolve(data); });
        stream.on('error', function (error) { return reject(error); });
    });
}
exports.readStream = readStream;
function readUrl(url) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var init, res, data;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    init = buildProxyOptions(url);
                    return [4, fetch(url, init)];
                case 1:
                    res = _a.sent();
                    return [4, res.text()];
                case 2:
                    data = _a.sent();
                    if (!res.ok) {
                        throw new Error("Error on fetch from url(" + url + "): " + res.status + ", " + data);
                    }
                    return [2, data];
            }
        });
    });
}
exports.readUrl = readUrl;
function noProxy(url) {
    var e_1, _a;
    if (process.env.NO_PROXY) {
        try {
            for (var _b = tslib_1.__values(process.env.NO_PROXY.split(/[, ]+/)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var domain = _c.value;
                if (url.hostname.endsWith(domain)) {
                    return true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    return false;
}
function buildProxyOptions(url) {
    var parsedUrl = new url_1.URL(url);
    var proxyUrl;
    if (!noProxy(parsedUrl)) {
        if (parsedUrl.protocol === 'http:' && process.env.HTTP_PROXY) {
            proxyUrl = new url_1.URL(process.env.HTTP_PROXY);
        }
        else if (parsedUrl.protocol === 'https:' && process.env.HTTPS_PROXY) {
            proxyUrl = new url_1.URL(process.env.HTTPS_PROXY);
        }
    }
    if (proxyUrl) {
        var agentOptions = {};
        agentOptions.protocol = proxyUrl.protocol;
        agentOptions.host = proxyUrl.hostname;
        agentOptions.port = proxyUrl.port;
        if (proxyUrl.username) {
            agentOptions.auth = proxyUrl.username + ':' + proxyUrl.password;
        }
        return { agent: https_proxy_agent_1.default(agentOptions) };
    }
    return undefined;
}
function globFiles(pattern, options) {
    return new Promise(function (resolve, reject) {
        glob_1.default(pattern, options || {}, function (err, matches) {
            if (err) {
                reject(err);
            }
            else {
                resolve(matches);
            }
        });
    });
}
exports.globFiles = globFiles;
//# sourceMappingURL=utils.js.map